#ifndef INFO_HEADER
#define INFO_HEADER

#include "Absyn.h"
#include <memory>
#include <unordered_map>
using namespace std;
class VarInfo;
class FunctionInfo;
class ClassInfo;
class Binding;
class Type;
using VarInfoPtr = shared_ptr<VarInfo>;
using FunctionInfoPtr = shared_ptr<FunctionInfo>;
using ClassInfoPtr = shared_ptr<ClassInfo>;
using BindingPtr = shared_ptr<Binding>;

#include "InfoList.h"

class VarInfo {
public:
	VarInfo(string n, int l = -1, Type* type = NULL) : name(n), type(type), lineLocation(l) {}
	VarInfo(PIdent* ident, Type* type = NULL) : VarInfo(ident->string_, ident->integer_, type) {}
	VarInfo(Ar* arg) : VarInfo(arg->pident_, arg->type_) {}

	string name;
	Type *type;
	int lineLocation;

	bool operator<(const VarInfo &other) const {
		return name != other.name ? name < other.name : type < other.type;
	}
};


class FunctionInfo : public VarInfo {
public:
	Block * block;
	vector<VarInfoPtr> arguments;
	weak_ptr<ClassInfo> klass;
	FunctionInfo(FuncDef *expr, ClassInfoPtr klass = nullptr);
	FunctionInfo(PIdent *ident, ClassInfoPtr klass = nullptr) : VarInfo(ident), block(NULL), klass(klass) {};
};


class ClassInfo : public VarInfo
{
	ClassInfoPtr parent;
public:
	InfoList<FunctionInfo> functions;
	InfoList<VarInfo> variables;

	ClassInfo(PIdent *ident, ClassInfoPtr parent = nullptr);

	ClassInfoPtr getParent() const { return parent; };
};


class Binding {
	InfoList<VarInfo> variables;
	BindingPtr parent;
	Binding(BindingPtr parent);
};


class Scope {
public:
	InfoList<ClassInfo> classes;
	InfoList<FunctionInfo> functions;
	ClassInfoPtr currentClass;
	Scope();
};

#endif